/*********************
callbacks.h           
	by logotherapy
*********************/
#include <libgnome/gnome-sound.h>
#include <libgnomeui/gnome-canvas.h>
#include <libgnomeui/gnome-canvas-text.h>
#include <libgnomeui/gnome-file-entry.h>
#include <libgnomeui/gnome-font-picker.h>
#include <libgnomeui/gtk-clock.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <stdio.h>



/*********************
DEFINE
*********************/
#define GTYPING_VERSION "0.4"

// character
#define GTYPING_VW_LENGTH 6
#define GTYPING_NM_LENGTH 14
#define GTYPING_OFF_SHIFT 0
#define GTYPING_KEYS_MAX 103
#define GTYPING_SUPPORT_LANG 5
// character flag
typedef enum {
	GTYPING_KEYS,
	GTYPING_SPACE,
	GTYPING_BACKSPACE,
	GTYPING_RETURN,
	GTYPING_ESCAPE,
	GTYPING_ETC
} KeyFlag;

// text
#define GTYPING_TEXT_LINE_MAX 2048
#define GTYPING_TEXT_PAGE 5

// game
#define GTYPING_GAME_UNIT 10
// game state
typedef enum {
	GTYPING_STOP, // move stop
	GTYPING_MOVE, // move 
} GameState;



/*********************
STRUCTURE             
*********************/
typedef struct
{
	gchar v[GTYPING_VW_LENGTH], w[GTYPING_VW_LENGTH];
} lang_ts;



typedef struct
{
	KeyFlag flag;
	gint left,right,top,bottom;
    GtkWidget *b, *l, *e;
	gchar n[GTYPING_NM_LENGTH], m[GTYPING_NM_LENGTH];
	lang_ts lang[GTYPING_SUPPORT_LANG];
} key_ts;



typedef struct
{
	GnomeCanvasItem *gci;
	gchar text[20];
	gdouble x;
	gdouble y;
	GameState state;
} game_ts;



/*********************
CALLBACKS
*********************/
// character
void key_b_button_press_event (GtkButton *button, GdkEventButton *event, key_ts *gp_key);
void key_b_button_release_event (GtkButton *button, GdkEventButton *event, key_ts *gp_key);
void lang_mi_activate (GtkMenuItem *menuitem, gint gi_lang);
void user_e_changed (GtkEditable *editable, key_ts *gp_key);
void user_tb_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void user_b_clicked (GtkButton *button, gpointer user_data);

// text
void text_ce_activate (GtkEditable *editable, gpointer user_data);
void text_a_value_changed (GtkWidget *widget, gpointer user_data);
void text_gfp_font_set (GnomeFontPicker *gnomefontpicker, gchar *font_name, gpointer user_data);
void text_e_insert_text (GtkEditable *editable, gchar *new_text, gint new_text_length, gint *position, gpointer user_data);
void text_e_delete_text (GtkEditable *editable, gint start_pos, gint end_pos, gpointer user_data);

// game
void game_e_activate (GtkEditable *editable, gpointer user_data);

// main
gboolean main_w_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean main_w_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
gboolean main_w_key_release_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);



/*********************
PROTOTYPE             
*********************/
// character
GtkWidget* create_character_v (void);
void key_load_main (gchar *gc_lang);
void key_load_lang (gchar *gc_lang, gint gi_lang);
void key_set_user_key ();
void key_init (GtkStyle *keyboard_s);
void key_press_shift ();
void key_release_shift ();

// text
GtkWidget* create_text_table (void);
void text_te_init ();

// game
GtkWidget* create_game_table (void);

// about
GtkWidget* create_about_l (void);

// main
void create_main_w (void);
